# -*- coding: utf-8 -*-
import calendar
import math
import time
import requests
import re


class Token:
    """ Token (Google Translate Token)
    Generate the current token key and allows generation of tokens (tk) with it
    Python version of `token-script.js` itself from translate.google.com
    """

    SALT_1 = "+-a^+6"
    SALT_2 = "+-3^+b+-f"


    def __init__(self, max_retries=5, country_domain_name='com'):
        """
        :param max_retries: int Number of conducted token key retrieval retries
            before error will be thrown """
        self.token_key = None
        self.country_domain_name = country_domain_name
        self._max_retries = max_retries


    def calculate_token(self, text, seed=None):
        """ Calculate the request token (`tk`) of a string
        :param text: str The text to calculate a token for
        :param seed: str The seed to use. By default this is the number of hours since epoch
        """

        if seed is None:
            if self.token_key is None:
                seed = self._get_token_key()
            else:
                seed = self.token_key

        [first_seed, second_seed] = seed.split(".")

        try:
            d = bytearray(text.encode('UTF-8'))
        except UnicodeDecodeError:
            # This will probably only occur when d is actually a str containing UTF-8 chars, which means we don't need
            # to encode.
            d = bytearray(text)

        a = int(first_seed)
        for value in d:
            a += value
            a = self._work_token(a, self.SALT_1)
        a = self._work_token(a, self.SALT_2)
        a ^= int(second_seed)
        if 0 > a:
            a = (a & 2147483647) + 2147483648
        a %= 1E6
        a = int(a)
        return str(a) + "." + str(a ^ int(first_seed))

    def _get_token_key(self, retry=0):
        response = requests.get("https://translate.google.%s/" % self.country_domain_name)
        tkk_expr = re.search("(tkk:.*?),", response.text)

        if tkk_expr is None:
            if retry == self._max_retries:
                raise ValueError(
                    "Unable to find token seed! Did https://translate.google.com change?"
                )
            else:
                return self._get_token_key(retry=retry + 1)


        tkk_expr = tkk_expr.group(1)
        try:
            # Grab the token directly if already generated by function call
            result = re.search(r"\d{6}\.[0-9]+", tkk_expr).group(0)
        except AttributeError:
            # Generate the token using algorithm
            timestamp = calendar.timegm(time.gmtime())
            hours = int(math.floor(timestamp / 3600))
            a = re.search(r"a\\\\x3d(-?\d+);", tkk_expr).group(1)
            b = re.search(r"b\\\\x3d(-?\d+);", tkk_expr).group(1)

            result = str(hours) + "." + str(int(a) + int(b))

        self.token_key = result
        return result

    """ Functions used by the token calculation algorithm """
    def _rshift(self, val, n):
        return val >> n if val >= 0 else (val + 0x100000000) >> n

    def _work_token(self, a, seed):
        for i in range(0, len(seed) - 2, 3):
            char = seed[i + 2]
            d = ord(char[0]) - 87 if char >= "a" else int(char)
            d = self._rshift(a, d) if seed[i + 1] == "+" else a << d
            a = a + d & 4294967295 if seed[i] == "+" else a ^ d
        return a


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

exec(open('gtts_token/version.py').read())

setup(
    name='gTTS-token',
    version=__version__,
    author='Boudewijn van Groos',
    author_email='boudewijn@vangroos.nl',
    url='https://github.com/boudewijn26/gTTS-token',
    packages=['gtts_token'],
    license='MIT',
    description='Calculates a token to run the Google Translate text to speech',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    install_requires=[
        "requests"
    ],
    classifiers=[
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Unix',
          'Operating System :: POSIX',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Topic :: Software Development :: Libraries',
          'Topic :: Multimedia :: Sound/Audio :: Speech'
    ],
)
